import yaml
import json
from typing import Annotated, Type
from pydantic import BaseModel, Field
from langchain_core.tools import BaseTool
from ..utls import log


class MapYmlToJsonSchema(BaseModel):
    yml: str = Field(description="The YAML to map to JSON.")


class MapYmlToJsonTool(BaseTool):
    """
    This tool maps YAML to JSON.
    """

    name: str = "map_yml_to_json"
    description: str = "Converts YAML to JSON."
    args_schema: Type[BaseModel] = MapYmlToJsonSchema

    def _run(self, yml: str) -> str:
        """Use the tool"""
        log(f"{self.name} START.")
        data = yaml.safe_load(yml)
        output = json.dumps(data, indent=2)
        log(f"{self.name} END.")
        return output

    async def _arun(self) -> str:
        """Use the tool"""
        return self._run()



from typing import Tuple, Optional
from langchain_core.tools import tool
from ..utls import log, exec_sh_cmd


@tool
def validate_mermaid_md(
    file_path: str, output_file_path: Optional[str] = None
) -> Tuple[str, str]:
    """Validates a .md file that contains Mermaid in it, by converting it to an image using Mermaid CLI.

    Args:
      filepath: The file path of the .md containing Mermaidto validate.
      output_file_path: The output file path, for any artifacts generated by the validation operation. Defaults to None.

    Returns:
      Tuple[str, str]: A tuple, with the first value being the validation output and the second the error, if there is one.
    """

    cmd = f"mmdc -i {file_path} -o {output_file_path or "tmp.md"}"

    log(f"validate_mermaid_md START. file_path: {file_path}, cmd: {cmd}")

    return exec_sh_cmd(cmd)



import os
from typing import List, Dict
from langchain_core.tools import tool
from ..utls import log, traverse_folder


@tool
def write_contents_to_file(file_path: str, file_content: str) -> bool:
    """Writes text contents to a specified file path or overwrites existing file contents. Creates the file if it doesn't exist already.

    Args:
      filepath: The file path of the file to write.
      content: The contents to write to file.

    Returns:
      str: A message indicating the file was written to.
    """

    log(f"write_contents_to_file START. file_path: {file_path}")

    if os.path.dirname(file_path):
        os.makedirs(os.path.dirname(file_path), exist_ok=True)

    with open(file_path, "w", encoding="utf-8") as f:
        f.write(file_content)

    log(f"write_contents_to_file END. file_path: {file_path}")

    return True


@tool
def read_file_contents(file_path: str) -> str:
    """Reads the contents of a file.

    Args:
      file_path: The path to the file to read.

    Returns:
      str: The contents of the file.
    """

    log(f"read_file_contents START. file_path: {file_path}")

    with open(file_path, "r", encoding="utf-8") as f:
        return f.read()

@tool
def walk_folder(folder_path: str, ignore_folders: List[str]) -> Dict[str, List[str]]:
    """Traverses a folder and returns a dictionary with the folder path as the key and a list of files as the value."""
    return traverse_folder(folder_path=folder_path, ignore_folders=ignore_folders)



import os
from typing import List, Dict
from .logger_utls import log


def traverse_folder(
    folder_path: str, ignore_folders: List[str], ignore_extensions: List[str] = None
) -> Dict[str, List[str]]:

    log(f"{traverse_folder.__name__} START. folder_path: {folder_path}")

    file_dict = {}

    for root, dirs, files in os.walk(folder_path):
        dirs[:] = [d for d in dirs if d not in ignore_folders]

        if ignore_extensions:
            files = [
                f
                for f in files
                if not any(f.endswith(ext) for ext in ignore_extensions)
            ]

        file_dict[root] = files

    log(f"{traverse_folder.__name__} END.")

    return file_dict



from typing import Tuple
from langchain_core.tools import tool
import subprocess
from .logger_utls import log


def exec_sh_cmd(cmd: str) -> Tuple[str, str]:
    """Executes a bash command.

    Args:
      cmd: The command to exectute.

    Returns:
      Tuple[str, str]: A tuple, with the first value being the output and the second the error, if there is one.
    """

    log(f"{exec_sh_cmd.__name__} START. cmd: {cmd}.")

    try:
        result = subprocess.run(
            cmd,
            shell=True,
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        output = result.stdout.decode("utf-8").strip()
        err = result.stderr.decode("utf-8").strip()

        log(f"{exec_sh_cmd.__name__} END.")

        return output, err
    except subprocess.CalledProcessError as e:
        log(f"{exec_sh_cmd.__name__} ERROR. cmd: {cmd}, error: {str(e)}")
        return None, str(e)



from typing import Tuple, Optional
from langchain_core.tools import tool
from ..utls import log, exec_sh_cmd


@tool
def dotnet_build(proj_path: str) -> Tuple[str, str]:
    """Builds a project using the .NET CLI. Runs `dotnet build` in the project directory.

    Args:
      proj_path: The path to the .csproj file to build.

    Returns:
      Tuple[str, str]: A tuple, with the first value being the build output and the second the error, if there is one.
    """

    cmd = f"dotnet build {proj_path}"

    log(f"dotnet_build START. proj_path: {proj_path}, cmd: {cmd}")

    return exec_sh_cmd(cmd)



from langchain_openai import AzureChatOpenAI, AzureOpenAIEmbeddings
from .config_loader import load_openai_config
import os

openai_config = load_openai_config()


class ModelFactory:
    @staticmethod
    def create():
        llm = AzureChatOpenAI(**openai_config)
        return llm


class EmbeddingFactory:
    @staticmethod
    def create():
        azure_embedding = AzureOpenAIEmbeddings(
            deployment="text-embedding-ada-002",
            azure_endpoint=os.getenv("AZURE_OPENAI_ENDPOINT"),
            api_key=os.getenv("AZURE_OPENAI_API_KEY"),
        )
        return azure_embedding
